Object subclass: #LinksChangeForProperty
	instanceVariableNames: 'model property initialValue finalValue hasInitialValue hasFinalValue lockAttemptStatus '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksChangeForProperty methodsFor: 'locking' stamp: 'pdf 3/9/2000 08:14'!
lock

  (lockAttemptStatus ~= #notTried and: [lockAttemptStatus ~= #releasedLock]) 
        ifTrue: [self halt: 'already tried lock.'. ^false].
  (model isLockedProperty: property) ifTrue: [lockAttemptStatus := #alreadyLocked. ^false].
  model property: property lock: true.
  lockAttemptStatus := #gotLock.
  ^true.! !

!LinksChangeForProperty methodsFor: 'locking' stamp: 'pdf 3/9/2000 08:14'!
unlock

  lockAttemptStatus ~= #gotLock ifTrue: [^false].
  model property: property lock: false.
  lockAttemptStatus := #releasedLock.
  ^true.! !


!LinksChangeForProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:14'!
getFinalValue
  (hasFinalValue ~= false) ifTrue: [finalValue := model property: property].
  ! !

!LinksChangeForProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:14'!
hasFinalValue: aBoolean
   hasFinalValue := aBoolean.
! !

!LinksChangeForProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:14'!
hasInitialValue: aBoolean
  hasInitialValue := aBoolean.
! !

!LinksChangeForProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:14'!
model: aModel property: aSymbol
  lockAttemptStatus := #notTried.
  model := aModel.
  property := aSymbol.
  (hasInitialValue ~= false) ifTrue: [initialValue := aModel property: property].
  "finalValue will be filled in later"
  finalValue := nil. 
  ^self.
  ! !

!LinksChangeForProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:14'!
restoreFinalValue
  (hasFinalValue ~= false) ifTrue: [model property: property value: finalValue].
  ! !

!LinksChangeForProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:14'!
restoreInitialValue
  (hasInitialValue ~= false) ifTrue: [model property: property value: initialValue].
  ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksChangeForProperty class
	instanceVariableNames: ''!

!LinksChangeForProperty class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:14'!
model: aModel property: aSymbol
  ^self new model: aModel property: aSymbol! !


Object subclass: #LinksEvent
	instanceVariableNames: 'type cursorPoint buttons keyValue source processed '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksEvent methodsFor: 'initialization' stamp: 'pdf 3/10/2000 11:13'!
initialize
	type := #unknown.
	cursorPoint := 0 @ 0.
	buttons := 0.
	keyValue := 0.
	source := nil.
	processed := false! !


!LinksEvent methodsFor: 'accessing' stamp: 'pdf 3/9/2000 07:57'!
buttons
	"Return the a word encoding the mouse and modifier buttons for this event."

	^ buttons! !

!LinksEvent methodsFor: 'accessing' stamp: 'pdf 3/10/2000 11:12'!
processed
  ^processed! !

!LinksEvent methodsFor: 'accessing' stamp: 'pdf 3/10/2000 11:12'!
processed: aBoolean
  processed := aBoolean! !

!LinksEvent methodsFor: 'accessing' stamp: 'pdf 3/9/2000 07:57'!
source
	"Return the source that originated this event."

	^ source! !

!LinksEvent methodsFor: 'accessing' stamp: 'pdf 3/9/2000 07:57'!
type
	"Return a symbol indicating the type this event."

	^ type! !


!LinksEvent methodsFor: 'classification' stamp: 'pdf 3/9/2000 07:57'!
isKeystroke

	^ type == #keystroke! !

!LinksEvent methodsFor: 'classification' stamp: 'pdf 3/9/2000 07:57'!
isMouse

	^ (type == #mouseMove) | (type == #mouseDown) | (type == #mouseUp)! !

!LinksEvent methodsFor: 'classification' stamp: 'pdf 3/9/2000 07:57'!
isMouseDown

	^ type == #mouseDown! !

!LinksEvent methodsFor: 'classification' stamp: 'pdf 3/9/2000 07:57'!
isMouseMove

	^ type == #mouseMove! !

!LinksEvent methodsFor: 'classification' stamp: 'pdf 3/9/2000 07:57'!
isMouseUp

	^ type == #mouseUp! !


!LinksEvent methodsFor: 'equality' stamp: 'pdf 3/9/2000 07:57'!
= aAWEvent

	(aAWEvent isKindOf: self class) ifFalse: [^ false].
	type = aAWEvent type ifFalse: [^ false].
	cursorPoint = aAWEvent cursorPoint ifFalse: [^ false].
	buttons = aAWEvent buttons ifFalse: [^ false].
	keyValue = aAWEvent keyValue ifFalse: [^ false].
     source = aAWEvent source ifFalse: [^false].
	^ true
! !

!LinksEvent methodsFor: 'equality' stamp: 'pdf 3/9/2000 07:57'!
hash

	^ type hash + cursorPoint hash + buttons hash + keyValue hash + source hash
! !


!LinksEvent methodsFor: 'mouse' stamp: 'pdf 3/9/2000 07:57'!
anyButtonPressed
	"Answer true if any mouse button is being pressed."

	^ buttons anyMask: 7! !

!LinksEvent methodsFor: 'mouse' stamp: 'pdf 3/9/2000 07:57'!
blueButtonPressed
	"Answer true if the blue mouse button is being pressed. This is the third mouse button or cmd+click on the Mac."

	^ buttons anyMask: 1! !

!LinksEvent methodsFor: 'mouse' stamp: 'pdf 3/9/2000 07:57'!
cursorPoint
	"Answer the location of the cursor's hotspot when this event occured."

	^ cursorPoint! !

!LinksEvent methodsFor: 'mouse' stamp: 'pdf 3/9/2000 07:57'!
redButtonPressed
	"Answer true if the red mouse button is being pressed. This is the first mouse button."

	^ buttons anyMask: 4! !

!LinksEvent methodsFor: 'mouse' stamp: 'pdf 3/9/2000 07:57'!
transformedBy: aMorphicTransform
	"Return a copy of the receiver transformed by the given transformation."

	aMorphicTransform isIdentity ifTrue: [^ self].  "no transformation needed"
	^ self copy setCursorPoint: (aMorphicTransform globalPointToLocal: cursorPoint)
! !

!LinksEvent methodsFor: 'mouse' stamp: 'pdf 3/9/2000 07:57'!
yellowButtonPressed
	"Answer true if the yellow mouse button is being pressed. This is the second mouse button or option+click on the Mac."

	^ buttons anyMask: 2! !


!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
anyModifierKeyPressed
	"ignore, however, the shift keys 'cause that's not REALLY a command key "

	^ self buttons anyMask: 16r70	"cmd | opt | ctrl"! !

!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
commandKeyPressed
	"Answer true if the command key on the keyboard was being held down when this event occurred."

	^ buttons anyMask: 64! !

!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
controlKeyPressed
	"Answer true if the control key on the keyboard was being held down when this event occurred."

	^ buttons anyMask: 16! !

!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
keyCharacter
	"Answer the character corresponding this keystroke. This is defined only for keystroke events."

	^ keyValue asCharacter! !

!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
keyValue
	"Answer the ascii value for this keystroke. This is defined only for keystroke events."

	^ keyValue! !

!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
macOptionKeyPressed
	"Answer whether the option key on the Macintosh keyboard was being held down when this event occurred. Macintosh specific."

	Preferences macOptionKeyAllowed ifFalse: [self notify: 'Portability note:
AWEvent>>macOptionKeyPressed is not portable.
Please use AWEvent>>yellowButtonPressed instead!!'].
	^ buttons anyMask: 32! !

!LinksEvent methodsFor: 'keyboard' stamp: 'pdf 3/9/2000 07:57'!
shiftPressed
	"Answer true if the shift key on the keyboard was being held down when this event occurred."

	^ buttons anyMask: 8
! !


!LinksEvent methodsFor: 'printing' stamp: 'pdf 3/9/2000 07:57'!
printOn: aStream

	aStream nextPut: $[.
	aStream nextPutAll: self cursorPoint printString; space.
	aStream nextPutAll: type.
	self isKeystroke ifTrue: [
		self controlKeyPressed ifTrue: [
			aStream nextPutAll: ' ''^'.
			aStream nextPut: (keyValue + $a asciiValue - 1) asCharacter.
		] ifFalse: [
			aStream nextPutAll: ' '''.
			aStream nextPut: self keyCharacter.
		].
		aStream nextPut: $'.
	].
	aStream nextPut: $].! !

!LinksEvent methodsFor: 'printing' stamp: 'pdf 3/9/2000 07:57'!
storeOn: aStream

	aStream nextPutAll: type.
	aStream space.
	cursorPoint x storeOn: aStream.
	aStream space.
	cursorPoint y storeOn: aStream.
	aStream space.
	buttons storeOn: aStream.
	aStream space.
	keyValue storeOn: aStream.
! !


!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setButtons: mask
	buttons _ mask! !

!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setCursorPoint: aPoint
	cursorPoint _ aPoint.! !

!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setKeyValue: keyVal mousePoint: aPoint buttons: anInteger source: aSource

	type _ #keystroke.
	cursorPoint _ aPoint.
	buttons _ anInteger.
	keyValue _ keyVal.
	source _ aSource.
! !

!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setMousePoint: aPoint buttons: anInteger lastEvent: lastEvent source: aSource

	cursorPoint _ aPoint.
	buttons _ anInteger.
	keyValue _ 0.
	source _ aSource.

     lastEvent isNil ifTrue: [type := #mouseMove. ^self].

	self anyButtonPressed ifTrue: [
		lastEvent anyButtonPressed
			ifTrue: [type _ #mouseMove]
			ifFalse: [type _ #mouseDown].
	] ifFalse: [
		lastEvent anyButtonPressed
			ifTrue: [type _ #mouseUp]
			ifFalse: [type _ #mouseMove]].

! !

!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setSource: aSource
	"Set the source that originated this event."

	source _ aSource.
! !

!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setType: newType

	type _ newType.! !

!LinksEvent methodsFor: 'private' stamp: 'pdf 3/9/2000 07:57'!
setType: t cursorPoint: p buttons: b keyValue: k

	type _ t.
	cursorPoint _ p.
	buttons _ b.
	keyValue _ k.
	source _ nil.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksEvent class
	instanceVariableNames: ''!

!LinksEvent class methodsFor: 'instance creation' stamp: 'pdf 3/9/2000 07:57'!
new

	^ super new initialize! !

!LinksEvent class methodsFor: 'instance creation' stamp: 'pdf 3/9/2000 07:57'!
newWorldExtent: aPoint
	"Answer an event that records a WorldMorph window size change."

	^ self basicNew setType: #worldExtent
		cursorPoint: aPoint
		buttons: 0
		keyValue: 0
! !

!LinksEvent class methodsFor: 'instance creation' stamp: 'pdf 3/9/2000 07:57'!
readFrom: aStream
	"Read a AWEvent from the given stream."

	| type x y buttons keyValue typeString c |
	typeString _ String streamContents:
		[:s |   [(c _ aStream next) isLetter] whileTrue: [s nextPut: c]].
	typeString = 'mouseMove'
		ifTrue: [type _ #mouseMove  "fast treatment of common case"]
		ifFalse: [type _ typeString asSymbol].

	x _ Integer readFrom: aStream.
	aStream skip: 1.
	y _ Integer readFrom: aStream.
	aStream skip: 1.

	buttons _ Integer readFrom: aStream.
	aStream skip: 1.

	keyValue _ Integer readFrom: aStream.

	^ self basicNew setType: type
		cursorPoint: x@y
		buttons: buttons
		keyValue: keyValue
! !


Object subclass: #LinksHold
	instanceVariableNames: 'widget offset '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!
!LinksHold commentStamp: 'pdf 3/9/2000 15:21' prior: 0!
Generic class to represetn holding a widget at an offset. Offset might be specific position reletive towidget bounds, or might be a symbol or object indicating a logical position on the widget ($bottomRight, #secondHandle, 5) that is meaningful to the widget.!


!LinksHold methodsFor: 'accessing' stamp: 'pdf 3/10/2000 00:19'!
getID
  "bad kludge for now"
  ^widget getID! !

!LinksHold methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:19'!
offset
  ^offset! !

!LinksHold methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:20'!
offset: anOffset
  offset := anOffset! !

!LinksHold methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:28'!
position
  ^widget positionForOffset: offset.! !

!LinksHold methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:19'!
widget
  ^widget! !

!LinksHold methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:19'!
widget: aWidget
  widget := aWidget! !


Inspector subclass: #LinksInspectorForModel
	instanceVariableNames: 'keyArray '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksInspectorForModel methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:15'!
fieldList
	^ keyArray collect: [ :key | key printString ]! !

!LinksInspectorForModel methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:15'!
inspect: aLinksModel
	"Initialize the receiver so that it is inspecting aLinksModel. There is no 
	current selection."

	self initialize.
	(aLinksModel isKindOf: LinksModelPrototype) ifFalse:
		[^ self error: 'LinksInspectorOnModels can only inspect LinksModels' ].
	object _ aLinksModel.
	contents _ ''.
	self calculateKeyArray! !


!LinksInspectorForModel methodsFor: 'selecting' stamp: 'pdf 3/9/2000 08:15'!
calculateKeyArray
	"Recalculate the KeyArray from the object being inspected"

	| sortedKeys |
	sortedKeys _ SortedCollection new: object propertyDefinitions size.
	sortedKeys sortBlock: [ :x :y |
		(((x isKindOf: String) & (y isKindOf: String))
		or: [(x isKindOf: Number) & (y isKindOf: Number)])
			ifTrue: [ x < y]
			ifFalse: [ (x class = y class)
				ifTrue: [ x printString < y printString]
				ifFalse: [ x class name < y class name ] ] ].
	object propertyDefinitions keysDo:
		[ :aKey | sortedKeys add: aKey. ].
	keyArray _ sortedKeys asArray.
	selectionIndex _ 0.
! !

!LinksInspectorForModel methodsFor: 'selecting' stamp: 'pdf 3/9/2000 08:15'!
contentsIsString
	"Hacked so contents empty when deselected"

	^ (selectionIndex = 0)! !

!LinksInspectorForModel methodsFor: 'selecting' stamp: 'pdf 3/9/2000 08:15'!
replaceSelectionValue: anObject
	| result |
	result := object property: (keyArray at: selectionIndex) value: anObject.
     object displayContentsIfInvalid.
     ^result
! !

!LinksInspectorForModel methodsFor: 'selecting' stamp: 'pdf 3/9/2000 08:15'!
selection

	selectionIndex = 0 ifTrue: [^ ''].
	^ object property: (keyArray at: selectionIndex)! !

!LinksInspectorForModel methodsFor: 'selecting' stamp: 'pdf 3/9/2000 08:15'!
selectionUnmodifiable

	^ selectionIndex <= 0 or: 
         [(object propertyDefinitions at: (keyArray at: selectionIndex)) storeSelector isNil].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksInspectorForModel class
	instanceVariableNames: ''!

!LinksInspectorForModel class methodsFor: 'testing' stamp: 'pdf 3/9/2000 08:15'!
test

 LinksInspectorForModel openOn: LinksModelPrototype new withEvalPane: false.! !


Model subclass: #LinksManagerForTransactions
	instanceVariableNames: 'domain transactionsInProgress transactionLog '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksManagerForTransactions methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 14:22'!
addTransactionToLog: aTransaction 
	"this is called by a transaction when it completes, which is either when 
	it is in progress and gets enough events, or if it was created as a 
	finished product and added to the manager complete."
	transactionLog add: aTransaction! !

!LinksManagerForTransactions methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 14:22'!
completeTransactionInProgress: aTransaction log: aBoolean

  transactionsInProgress remove: aTransaction.
  aBoolean ifTrue: [self addTransactionToLog: aTransaction].! !

!LinksManagerForTransactions methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 14:22'!
handleEvent: anEvent
  "return boolean indicating if used up event"
  | transactionHandlingEvent |
  "iterate over a copy because transactions might complete on event"
  "might need another layer of logic to arbitrate who gets what events"
  "for now, only allow one to absorb an event. Might want to give it to all..."
  transactionHandlingEvent := transactionsInProgress 
      detect: [:each | each handleEvent: anEvent] ifNone: [nil].
  ^transactionHandlingEvent notNil.! !

!LinksManagerForTransactions methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 14:22'!
newTransactionInProgress: aTransaction event: event
  "it is typical for an event to start a transaction, and then be passed to the transaction."
  transactionsInProgress add: aTransaction.
  ^event notNil 
	ifTrue: [aTransaction handleEvent: event] 
	ifFalse: [true].! !


!LinksManagerForTransactions methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:22'!
domain

  ^domain! !

!LinksManagerForTransactions methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:22'!
domain: aDomain

  domain := aDomain.! !


!LinksManagerForTransactions methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 14:22'!
initialize

  transactionsInProgress := OrderedCollection new.
  transactionLog := OrderedCollection new.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksManagerForTransactions class
	instanceVariableNames: ''!

!LinksManagerForTransactions class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 14:22'!
domain: aDomain

  ^super new initialize; domain: aDomain; yourself.! !


Model subclass: #LinksModelPrototype
	instanceVariableNames: 'lockedProperties uniqueID '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksModelPrototype methodsFor: 'initalizing-releasing' stamp: 'pdf 3/9/2000 08:11'!
generateUniqueID

  uniqueID := LinksUniqueID new.

! !

!LinksModelPrototype methodsFor: 'initalizing-releasing' stamp: 'pdf 3/9/2000 08:11'!
initialize

  self generateUniqueID.
  self unlockAllProperties.! !


!LinksModelPrototype methodsFor: 'inspecting' stamp: 'pdf 3/9/2000 08:11'!
displayContentsIfInvalid
  self domain notNil ifTrue: [self domain displayContentsIfInvalid].! !

!LinksModelPrototype methodsFor: 'inspecting' stamp: 'pdf 3/9/2000 15:03'!
inspect
  LinksInspectorForModel openOn: self withEvalPane: false.! !


!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/10/2000 08:30'!
isLockedProperty: aProperty 
	^self lockedProperties at: aProperty ifAbsent: [false]! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
property: aSymbol
  "return the value of this property"
  ^self perform: (self selectorToLoadProperty: aSymbol)! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
property: aProperty lock: aBoolean 
	^lockedProperties at: aProperty put: aBoolean! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 15:40'!
property: aSymbol value: aValue
  "set the value of this property"
  (self selectorToStoreProperty: aSymbol) isNil ifTrue: [^self].
  ^self perform: (self selectorToStoreProperty: aSymbol) with: aValue! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
propertyDefinitions
	^self class propertyDefinitions! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
selectorToLoadProperty: aSymbol
  ^self class selectorToLoadProperty: aSymbol! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
selectorToStoreProperty: aSymbol
  ^self class selectorToStoreProperty: aSymbol! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
uniqueID
  ^uniqueID! !

!LinksModelPrototype methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
unlockAllProperties
  lockedProperties := IdentityDictionary new.! !


!LinksModelPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:11'!
domain
  ^nil! !


!LinksModelPrototype methodsFor: 'xml streaming' stamp: 'pdf 3/10/2000 08:28'!
readXMLElement: element 
	| content newObject |
	self class propertyDefinitions do: [:each | each type ~= #OrderedCollection
			ifTrue: 
				[content := (element findSubElement: each name asLowercase) content.
				newObject := Compiler evaluate: content.
				self property: each symbol value: newObject]]! !

!LinksModelPrototype methodsFor: 'xml streaming' stamp: 'pdf 3/10/2000 00:18'!
writeXML: aXMLWriter 
	aXMLWriter tagStartOpen: 'model'.
     aXMLWriter tagStartParameter: 'ID' value: self getID printString.
     aXMLWriter tagStartParameter: 'class' value: self class printString.
	aXMLWriter tagStartClose.
	self writeXMLForProperties: aXMLWriter.
	aXMLWriter tagEnd: 'model'! !

!LinksModelPrototype methodsFor: 'xml streaming' stamp: 'pdf 3/10/2000 07:44'!
writeXML: aXMLWriter forProperty: aProperty 
	"Transcript show: aProperty name; cr."
     "aProperty name = 'uniqueID' ifTrue: [^self]."
	aXMLWriter tagStartSingleLine: aProperty name.
	aProperty type == #OrderedCollection
		ifTrue: [(self property: aProperty loadSelector)
				do: [:each | aXMLWriter contents: each getID storeString , ' ']]
		ifFalse: [aXMLWriter contents: (self property: aProperty loadSelector) storeString].
	aXMLWriter tagEndSingleLine: aProperty name! !

!LinksModelPrototype methodsFor: 'xml streaming' stamp: 'pdf 3/9/2000 23:45'!
writeXMLForProperties: aXMLWriter
  self class propertyDefinitions do: [:each | self writeXML: aXMLWriter forProperty: each].
  
  ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksModelPrototype class
	instanceVariableNames: 'propertyDefinitions '!

!LinksModelPrototype class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:11'!
new

  ^super new initialize! !


!LinksModelPrototype class methodsFor: 'property handling' stamp: 'pdf 3/10/2000 11:42'!
createPropertyDictionary
      | definition |
     propertyDefinitions := IdentityDictionary new.
     "super createPropertyDictionary".
     self propertyList do: [:each |
          definition := self perform: each.
		propertyDefinitions at: definition symbol put: definition].
! !

!LinksModelPrototype class methodsFor: 'property handling' stamp: 'pdf 3/9/2000 17:55'!
initialize
  "LinksModelPrototype initialize".
  Transcript show: 'Initializing ', self printString; cr.
  self createPropertyDictionary.
  self subclasses do: [:each | each initialize].
  ! !

!LinksModelPrototype class methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
propertyDefinitions
  ^propertyDefinitions! !

!LinksModelPrototype class methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
selectorToLoadProperty: aSymbol
  ^(self propertyDefinitions at: aSymbol) loadSelector.! !

!LinksModelPrototype class methodsFor: 'property handling' stamp: 'pdf 3/9/2000 08:11'!
selectorToStoreProperty: aSymbol
    ^(self propertyDefinitions at: aSymbol) storeSelector.! !

!LinksModelPrototype class methodsFor: 'property handling' stamp: 'pdf 3/10/2000 00:46'!
uniqueID
	^ LinksProperty
		name: 'uniqueID'
		symbol: #uniqueID
		loadSelector: #uniqueID
		storeSelector: nil
		hint: 'This is a unique ID for this model based on creation time.'
		type: #LinksUniqueID! !


!LinksModelPrototype class methodsFor: 'property defining' stamp: 'pdf 3/10/2000 00:42'!
propertyList
  "LinksModelPrototype initialize".
  ^#(uniqueID)
  "^super propertyList , #(newProp1 newProp2)"
! !


Object subclass: #LinksProperty
	instanceVariableNames: 'name symbol loadSelector storeSelector hint type '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksProperty methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:07'!
name: aName symbol: aSymbol loadSelector: aLoadSelector storeSelector: aStoreSelector hint: aHint type: aType 
	name := aName.
	symbol := aSymbol.
	loadSelector := aLoadSelector.
	storeSelector := aStoreSelector.
	hint := aHint.
	type := aType! !


!LinksProperty methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
loadSelector
  ^loadSelector! !

!LinksProperty methodsFor: 'accessing' stamp: 'pdf 3/10/2000 00:22'!
name
  ^name! !

!LinksProperty methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
storeSelector
  ^storeSelector! !

!LinksProperty methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
symbol
  ^symbol! !

!LinksProperty methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
type
  ^type! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksProperty class
	instanceVariableNames: ''!

!LinksProperty class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:07'!
name: aName symbol: aSymbol loadSelector: aLoadSelector storeSelector: aStoreSelector hint: aHint type: aType
  ^self new name: aName symbol: aSymbol loadSelector: aLoadSelector storeSelector: aStoreSelector hint: aHint type: aType  ! !


Object subclass: #LinksPropertyChange
	instanceVariableNames: 'model property initialValue finalValue hasInitialValue hasFinalValue lockAttemptStatus '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksPropertyChange methodsFor: 'locking' stamp: 'pdf 3/9/2000 22:26'!
lock

  (lockAttemptStatus ~= #notTried and: [lockAttemptStatus ~= #releasedLock]) 
        ifTrue: [self halt: 'already tried lock.'. ^false].
  (model isLockedProperty: property) ifTrue: [lockAttemptStatus := #alreadyLocked. ^false].
  model property: property lock: true.
  lockAttemptStatus := #gotLock.
  ^true.! !

!LinksPropertyChange methodsFor: 'locking' stamp: 'pdf 3/9/2000 22:26'!
unlock

  lockAttemptStatus ~= #gotLock ifTrue: [^false].
  model property: property lock: false.
  lockAttemptStatus := #releasedLock.
  ^true.! !


!LinksPropertyChange methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:26'!
getFinalValue
  (hasFinalValue ~= false) ifTrue: [finalValue := model property: property].
  ! !

!LinksPropertyChange methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:26'!
hasFinalValue: aBoolean
   hasFinalValue := aBoolean.
! !

!LinksPropertyChange methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:26'!
hasInitialValue: aBoolean
  hasInitialValue := aBoolean.
! !

!LinksPropertyChange methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:26'!
model: aModel property: aSymbol
  lockAttemptStatus := #notTried.
  model := aModel.
  property := aSymbol.
  (hasInitialValue ~= false) ifTrue: [initialValue := aModel property: property].
  "finalValue will be filled in later"
  finalValue := nil. 
  ^self.
  ! !

!LinksPropertyChange methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:26'!
restoreFinalValue
  (hasFinalValue ~= false) ifTrue: [model property: property value: finalValue].
  ! !

!LinksPropertyChange methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:26'!
restoreInitialValue
  (hasInitialValue ~= false) ifTrue: [model property: property value: initialValue].
  ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksPropertyChange class
	instanceVariableNames: ''!

!LinksPropertyChange class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 22:26'!
model: aModel property: aSymbol
  ^self new model: aModel property: aSymbol! !


StandardSystemController subclass: #LinksSystemController
	instanceVariableNames: 'lastEvent '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksSystemController methodsFor: 'basic control sequence' stamp: 'pdf 3/9/2000 07:56'!
controlActivity
	"Refer to the comment in Controller|controlActivity."
	super controlActivity.
	self processEvents.! !

!LinksSystemController methodsFor: 'basic control sequence' stamp: 'pdf 3/10/2000 11:50'!
processEvents
	"Process user input events from the local input devices."

	| newEvent mousePoint |
     mousePoint := sensor cursorPoint - view insetDisplayBox topLeft.

	newEvent _ LinksEvent new
		setMousePoint: mousePoint
		buttons: Sensor primMouseButtons
		lastEvent: lastEvent
		source: #console.

	lastEvent ~= newEvent
		ifTrue: 
			[lastEvent := newEvent.
			model handleEvent: newEvent].

	[sensor keyboardPressed] whileTrue: [
		newEvent _ LinksEvent new
			setKeyValue: sensor keyboard asciiValue
			mousePoint: mousePoint
			buttons: Sensor primMouseButtons
			source: #console.
		model handleEvent: newEvent].

  (status ~= #closed) 
     ifTrue: [view displayContentsIfInvalid].! !


StandardSystemView subclass: #LinksSystemView
	instanceVariableNames: 'testCanvas invalidRect '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksSystemView methodsFor: 'controller access' stamp: 'pdf 3/9/2000 08:19'!
defaultControllerClass 
	"Refer to the comment in View|defaultControllerClass."

	^LinksSystemController! !


!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
buildTestCanvas

	testCanvas := FormCanvas extent: self insetDisplayBox extent.
! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
displayBorder
	super displayBorder.
	self invalidate.! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
displayContents
     self isCollapsed ifTrue: [^self].
	invalidRect isNil ifTrue: [^ self].
	testCanvas isNil ifTrue: [self buildTestCanvas].
	testCanvas fillRectangle: invalidRect color: Color white.
	model displayOn: testCanvas invalidRect: invalidRect.
	testCanvas showAt: self insetDisplayBox topLeft invalidRects: (OrderedCollection with: invalidRect).
     self invalidRectReset.! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
displayContentsIfInvalid
	invalidRect isNil ifTrue: [^ self].
	self displayContents.
! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
displayView
	super displayView.
	self displayContents.! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
initialize

   super initialize.
  invalidRect := nil.
  
! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
invalidRectReset

   invalidRect := nil.
! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
invalidate

   invalidRect := 0 @ 0 extent: self insetDisplayBox extent.

  
! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
invalidate: aRect

   invalidRect := 
	invalidRect isNil ifTrue: [aRect copy] ifFalse: [invalidRect merge: aRect].! !

!LinksSystemView methodsFor: 'displaying' stamp: 'pdf 3/9/2000 07:56'!
resizeTo: aRectangle
	super resizeTo: aRectangle.
     self buildTestCanvas.! !


Model subclass: #LinksTransactionManager
	instanceVariableNames: 'domain transactionsInProgress transactionLog '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksTransactionManager methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 07:58'!
addTransactionToLog: aTransaction 
	"this is called by a transaction when it completes, which is either when 
	it is in progress and gets enough events, or if it was created as a 
	finished product and added to the manager complete."
	transactionLog add: aTransaction! !

!LinksTransactionManager methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 07:58'!
completeTransactionInProgress: aTransaction log: aBoolean

  transactionsInProgress remove: aTransaction.
  aBoolean ifTrue: [self addTransactionToLog: aTransaction].! !

!LinksTransactionManager methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 07:58'!
handleEvent: anEvent
  "return boolean indicating if used up event"
  | transactionHandlingEvent |
  "iterate over a copy because transactions might complete on event"
  "might need another layer of logic to arbitrate who gets what events"
  "for now, only allow one to absorb an event. Might want to give it to all..."
  transactionHandlingEvent := transactionsInProgress 
      detect: [:each | each handleEvent: anEvent] ifNone: [nil].
  ^transactionHandlingEvent notNil.! !

!LinksTransactionManager methodsFor: 'transaction handling' stamp: 'pdf 3/9/2000 07:58'!
newTransactionInProgress: aTransaction event: event
  "it is typical for an event to start a transaction, and then be passed to the transaction."
  transactionsInProgress add: aTransaction.
  ^event notNil 
	ifTrue: [aTransaction handleEvent: event] 
	ifFalse: [true].! !


!LinksTransactionManager methodsFor: 'accessing' stamp: 'pdf 3/9/2000 07:58'!
domain

  ^domain! !

!LinksTransactionManager methodsFor: 'accessing' stamp: 'pdf 3/9/2000 07:58'!
domain: aDomain

  domain := aDomain.! !


!LinksTransactionManager methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 07:58'!
initialize

  transactionsInProgress := OrderedCollection new.
  transactionLog := OrderedCollection new.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksTransactionManager class
	instanceVariableNames: ''!

!LinksTransactionManager class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 07:58'!
domain: aDomain

  ^super new initialize; domain: aDomain; yourself.! !


Model subclass: #LinksTransactionPrototype
	instanceVariableNames: 'manager isInProgress propertyChanges initialConditions finalConditions locksNeeded additionalLocksNeeded sourcesObserving '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksTransactionPrototype methodsFor: 'event handling' stamp: 'pdf 3/9/2000 15:38'!
handleEvent: event
  (self mightWantEvent: event) ifFalse: [^false].
  ^self processEvent: event.! !

!LinksTransactionPrototype methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:07'!
isSimpleMouseTracker
   ^true
   "subclasses should override this if they want to handle more sophisticated things, like keyboard input or other types of events. If they return true, #handleEvent: gets called with the raw event instead of the standard three phase tracking method".! !

!LinksTransactionPrototype methodsFor: 'event handling' stamp: 'pdf 3/9/2000 15:38'!
mightWantEvent: event
  ^(sourcesObserving includes: event source).! !

!LinksTransactionPrototype methodsFor: 'event handling' stamp: 'pdf 3/9/2000 15:38'!
processEvent: event
  "event is not ignorable so do something about it"
  "should return true in subclass if absorbs event, false if don't really want event"
  ^self subclassResponsibility! !


!LinksTransactionPrototype methodsFor: 'tracking' stamp: 'pdf 3/9/2000 08:07'!
finish
	^ self! !

!LinksTransactionPrototype methodsFor: 'tracking' stamp: 'pdf 3/9/2000 08:07'!
start! !

!LinksTransactionPrototype methodsFor: 'tracking' stamp: 'pdf 3/9/2000 08:07'!
track
	^ self! !

!LinksTransactionPrototype methodsFor: 'tracking' stamp: 'pdf 3/9/2000 08:07'!
trackMousePhase: aTrackPhase anchorPoint: anchorPoint previousPoint: previousPoint nextPoint: nextPoint mouseDidMove: mouseDidMove rightButtonDown: rightButtonDown 
	"sublasses should override if needed -- this returns an action object to 
	continue tracking with, usually self."
	^ self! !


!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
actionChangesFile

^actionChangesFile! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
canUndo

^canUndo! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
description

  ^ '*command description*'.
! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
done

^done! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
fileChangedBeforeAction

^fileChangedBeforeAction! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:38'!
manager: aManager
  manager := aManager.! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:07'!
numberOfStoredLargeObjects

  ^  0
  "subclasses may override but only if they keep copies of large objects"
! !

!LinksTransactionPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:38'!
observeSource: aSource
  sourcesObserving add: aSource.! !




!LinksTransactionPrototype methodsFor: 'locking' stamp: 'pdf 3/9/2000 15:38'!
addLocks
  "lock all object properties needed to do task - returns true (all locks) or false (nothing locked)"
  ^true.! !

!LinksTransactionPrototype methodsFor: 'locking' stamp: 'pdf 3/9/2000 15:38'!
addLocksForPropertyChanges
  "lock all object properties needed to do task - returns true (all locks) or false (nothing locked)"
  propertyChanges detect: [:each | each lock not] ifNone: [ ^true].
  self removeLocks.
  ^false.
  ! !

!LinksTransactionPrototype methodsFor: 'locking' stamp: 'pdf 3/9/2000 15:38'!
removeLocks
  "unlock all obect properties needed to do task"
  ^self.! !

!LinksTransactionPrototype methodsFor: 'locking' stamp: 'pdf 3/9/2000 15:38'!
removeLocksForPropertyChanges
  "unlock all obect properties needed to do task"
  propertyChanges detect: [:each | each unlock not] ifNone: [ ^true].
  ^false.
  ! !


!LinksTransactionPrototype methodsFor: 'committing' stamp: 'pdf 3/9/2000 15:38'!
do
  "commit this transaction for the very first time"
  "usually just need to remove locks"
   self invalidateIfNeeded: #beforeDo.
   self getFinalValuesForPropertyChanges.
   self invalidateIfNeeded: #afterDo.
   self removeLocksForPropertyChanges.
   manager completeTransactionInProgress: self log: true.
  ! !

!LinksTransactionPrototype methodsFor: 'committing' stamp: 'pdf 3/9/2000 15:38'!
redo
  "commit this transaction again after having been undone"
  "should check that initial conditions still apply"
  self addLocksForPropertyChanges ifFalse: [^false].
  self invalidateIfNeeded: #beforeRedo.
  self restoreFinalValuesForPropertyChanges.
  self invalidateIfNeeded: #afterRedo.
  self removeLocksForPropertyChanges ifFalse: [^false].
  ^true.
  ! !

!LinksTransactionPrototype methodsFor: 'committing' stamp: 'pdf 3/9/2000 15:38'!
rollback
  "roll back a transaction in progress to the initial conditions. Can assume was never committed."
  "subclasses should override as needed"
  Transcript show: '**** rollback ***'; cr.
  self invalidateIfNeeded: #beforeRollback.
  self restoreInitialValuesForPropertyChanges.
  self invalidateIfNeeded: #afterRollback.
  self removeLocksForPropertyChanges.
  manager completeTransactionInProgress: self log: false.
  ! !

!LinksTransactionPrototype methodsFor: 'committing' stamp: 'pdf 3/9/2000 15:38'!
undo
  "undo the results of this transaction"
  "should check that final conditions still apply, or at the very least, returning to initial conditions makes sense."
  self addLocksForPropertyChanges ifFalse: [^false].
  self invalidateIfNeeded: #beforeUndo.
  self restoreInitialValuesForPropertyChanges.
  self invalidateIfNeeded: #afterUndo.
  self removeLocksForPropertyChanges ifFalse: [^false].
  ^true.

! !


!LinksTransactionPrototype methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 15:38'!
defineConditionsAndLocks
  "subclasses should usually override"
  ^self! !

!LinksTransactionPrototype methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 15:38'!
initialize
  isInProgress := false.
  propertyChanges := OrderedCollection new.
  additionalLocksNeeded := OrderedCollection new.
  sourcesObserving := OrderedCollection new.! !


!LinksTransactionPrototype methodsFor: 'property changing' stamp: 'pdf 3/9/2000 22:27'!
addPropertyChangeForModel: model property: aSymbol
     "add an initial condition to the list for undoing and locking"
	propertyChanges add:  
         (LinksPropertyChange model: model property: aSymbol).! !

!LinksTransactionPrototype methodsFor: 'property changing' stamp: 'pdf 3/9/2000 15:38'!
getFinalValuesForPropertyChanges
  propertyChanges do: [:each | each getFinalValue].! !

!LinksTransactionPrototype methodsFor: 'property changing' stamp: 'pdf 3/9/2000 15:38'!
restoreFinalValuesForPropertyChanges
  propertyChanges do: [:each | each restoreFinalValue].! !

!LinksTransactionPrototype methodsFor: 'property changing' stamp: 'pdf 3/9/2000 15:38'!
restoreInitialValuesForPropertyChanges
  propertyChanges do: [:each | each restoreInitialValue].! !


!LinksTransactionPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 15:38'!
invalidate
  manager domain invalidate.! !

!LinksTransactionPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 15:38'!
invalidate: aRect
  manager domain invalidate: aRect.! !

!LinksTransactionPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 15:38'!
invalidateIfNeeded: aSymbol
  "subclass might implement -- aSymbol can be things like #beforeRollback or #afterRollback"
  "ideally, this should not be needed as widgets should invalidate parts of domain when properties change"
  ^self
  ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksTransactionPrototype class
	instanceVariableNames: ''!

!LinksTransactionPrototype class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 15:38'!
manager: aManager

^self new initialize; manager: aManager; observeSource: #console; yourself.! !

!LinksTransactionPrototype class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:07'!
new

^super new initialize! !


LinksTransactionPrototype subclass: #LinksTransactionGesture
	instanceVariableNames: 'anchorPoint previousPoint nextPoint phase buttonTracked '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksTransactionGesture methodsFor: 'button handling' stamp: 'pdf 3/9/2000 08:08'!
finalButtonUpExpected
  "subclasses can override -- usually this button is expected to go up at end"
  ^buttonTracked! !

!LinksTransactionGesture methodsFor: 'button handling' stamp: 'pdf 3/9/2000 08:08'!
initialButtonDownExpected
  "subclasses can override -- usually a button down has already occured to start the transaction in progress, but it is customarily passed in as the first event anyway"
  ^buttonTracked! !


!LinksTransactionGesture methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:08'!
initialize

  super initialize.
  buttonTracked := #redButton.
  phase := #beforeStart.! !


!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
atEndPosition: event
  "the transaction definition is being finished at the nextPoint (which serves as the last point)"
  ^true! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
atNextIntermediatePosition: event
  "the transaction is progressing to nextPoint"
  ^true! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
atStartPosition: event
  "the transaction is starting to be defined from the anchor point"
  ^true! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
delta
  ^nextPoint - previousPoint
   ! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
didMouseMoveFromAnchor: point
  "see if in the end the object moved from the original"
  ^((point x - anchorPoint x) abs > self minMoveDistance)
        or: [(point y - anchorPoint y) abs > self minMoveDistance].! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
mightWantEvent: event 
	event isMouse ifTrue: [^ true].
     ^super mightWantEvent: event.
! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
minMoveDistance 
  ^2! !

!LinksTransactionGesture methodsFor: 'event handling' stamp: 'pdf 3/9/2000 08:08'!
processEvent: event 
	"event is not ignorable so do something about it"
     "might decide to return false if event terminates gesture (like close window?) but is not otherwise used..."
     "Transcript show: 'starting dispatch on event', event printString; cr."
	event isMouseDown
		ifTrue: 
			[phase ~= #beforeStart ifTrue: [self rollback. ^false].
              anchorPoint := event cursorPoint.
			previousPoint := anchorPoint.
			nextPoint := anchorPoint.
              phase := #startPosition.
  "Transcript show: 'about to call mouse down'; cr."
			self atStartPosition: event.
              ^true].
	event isMouseMove
		ifTrue: 
			[(phase ~= #startPosition and: [phase ~= #nextIntermediatePosition]) ifTrue: [self rollback. ^false].
			previousPoint := nextPoint.
			nextPoint := event cursorPoint.
               phase := #nextIntermediatePosition.
			self atNextIntermediatePosition: event.
			^true].
	event isMouseUp
		ifTrue: 
			[(phase ~= #startPosition and: [phase ~= #nextIntermediatePosition]) ifTrue: [self rollback. ^false].
			previousPoint := nextPoint.
			nextPoint := event cursorPoint.
              phase := #endPosition.
			self atEndPosition: event.
			^true].
     "some sort of unexpected mouse related event"
     Transcript show: 'unexpected event type'; cr.
	^ false ! !


LinksTransactionGesture subclass: #LinksTransactionNetworkMove
	instanceVariableNames: 'widgets network '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksTransactionNetworkMove methodsFor: 'event handling' stamp: 'pdf 3/9/2000 16:30'!
atEndPosition: event
  "the transaction definition is being finished at the nextPoint (which serves as the last point)"
  "Transcript show: 'atEnd'; cr."

  "handle case where mouse up and move are combined"
  self moveNetwork.

  "abandon if did not move enough"
  (self didMouseMoveFromAnchor: nextPoint) ifFalse: [self rollback. ^true].

  "now commit transaction"
  self do.
   ! !

!LinksTransactionNetworkMove methodsFor: 'event handling' stamp: 'pdf 3/9/2000 16:30'!
atNextIntermediatePosition: event
  "the transaction is progressing to nextPoint"
  "Transcript show: 'atInter'; cr."
  self moveNetwork.
! !

!LinksTransactionNetworkMove methodsFor: 'event handling' stamp: 'pdf 3/9/2000 17:22'!
atStartPosition: event
  "the transaction is starting to be defined from the anchor point"
  |widget|
  "Transcript show: 'atStart'; cr."

  "first need to find the widget, and then get locks, etc."
   widget := manager domain widgetUnderPoint: anchorPoint.

   "if don't find a widget -- give up..."
   widget isNil ifTrue: [self rollback. ^false].
  widgets := Set new.
  widget recurseLinks: widgets from: nil type: #solid.
  "Transcript show: 'got network'; cr."
   "have a network -- now record initial position and lock its position"
  widgets do: 
    [:each | self addPropertyChangeForModel: each property: #position.].
   self addLocksForPropertyChanges ifFalse: [self rollback. ^false].! !


!LinksTransactionNetworkMove methodsFor: 'supporting' stamp: 'pdf 3/9/2000 16:36'!
moveNetwork
  widgets do: [:each | each moveBy: self delta.].
   ! !


LinksTransactionGesture subclass: #LinksTransactionWidgetConnect
	instanceVariableNames: 'firstWidget secondWidget '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksTransactionWidgetConnect methodsFor: 'event handling' stamp: 'pdf 3/9/2000 15:50'!
atEndPosition: event
  | link |
"the transaction definition is being finished at the nextPoint (which serves as the last point)"
  "Transcript show: 'atEnd'; cr."

  "abandon if did not move enough"
  (self didMouseMoveFromAnchor: nextPoint) ifFalse: [self rollback. ^true].

   secondWidget := manager domain widgetUnderPoint: nextPoint.

   "if don't find a widget -- give up..."
   (secondWidget isNil or: [firstWidget == secondWidget]) ifTrue: [self rollback. ^false].

   "do this for now -- should be done as undoable"
   link := LinksWidgetLinkPrototype world: firstWidget world position: firstWidget position.
   link addHold: firstWidget offset: 0 @ 0.
   link addHold: secondWidget offset: 0 @ 0.
  "now commit transaction"
  self do.
   ! !

!LinksTransactionWidgetConnect methodsFor: 'event handling' stamp: 'pdf 3/9/2000 15:34'!
atNextIntermediatePosition: event
  "the transaction is progressing to nextPoint"
  "Transcript show: 'atInter'; cr."
  "do nothing"
! !

!LinksTransactionWidgetConnect methodsFor: 'event handling' stamp: 'pdf 3/9/2000 15:42'!
atStartPosition: event
  "the transaction is starting to be defined from the anchor point"
  "Transcript show: 'atStart'; cr."
  "first need to find the widget, and then get locks, etc."

   firstWidget := manager domain widgetUnderPoint: anchorPoint.

   "if don't find a widget -- give up..."
   firstWidget isNil ifTrue: [self rollback. ^false].
  "Transcript show: 'got first widget'; cr."
   "have a widget -- now record initial position and lock its position"
  " self addPropertyChangeForModel: firstWidget property: #connections."
   self addLocksForPropertyChanges ifFalse: [self rollback. ^false].
   ! !

!LinksTransactionWidgetConnect methodsFor: 'event handling' stamp: 'pdf 3/9/2000 16:46'!
rollback
  "self halt."
  super rollback.! !


LinksTransactionGesture subclass: #LinksTransactionWidgetMove
	instanceVariableNames: 'widget '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksTransactionWidgetMove methodsFor: 'supporting' stamp: 'pdf 3/9/2000 16:14'!
moveWidget
  widget moveBy: self delta.
   ! !


!LinksTransactionWidgetMove methodsFor: 'event handling' stamp: 'pdf 3/9/2000 16:14'!
atEndPosition: event
  "the transaction definition is being finished at the nextPoint (which serves as the last point)"
  "Transcript show: 'atEnd'; cr."

  "handle case where mouse up and move are combined"
  self moveWidget.

  "abandon if did not move enough"
  (self didMouseMoveFromAnchor: nextPoint) ifFalse: [self rollback. ^true].

  "now commit transaction"
  self do.
   ! !

!LinksTransactionWidgetMove methodsFor: 'event handling' stamp: 'pdf 3/9/2000 16:14'!
atNextIntermediatePosition: event
  "the transaction is progressing to nextPoint"
  "Transcript show: 'atInter'; cr."
  self moveWidget.
! !

!LinksTransactionWidgetMove methodsFor: 'event handling' stamp: 'pdf 3/9/2000 16:14'!
atStartPosition: event
  "the transaction is starting to be defined from the anchor point"
 " Transcript show: 'atStart'; cr."
  "first need to find the widget, and then get locks, etc."

   widget := manager domain widgetUnderPoint: anchorPoint.

   "if don't find a widget -- give up..."
   widget isNil ifTrue: [self rollback. ^false].
  "Transcript show: 'got widget'; cr."
   "have a widget -- now record initiaia position and lock its position"
   self addPropertyChangeForModel: widget property: #position.
   self addLocksForPropertyChanges ifFalse: [self rollback. ^false].
   ! !


Object subclass: #LinksUniqueID
	instanceVariableNames: 'domainSymbol creatorSymbol timestamp creationDateAndTime randomNumber version '
	classVariableNames: 'CreatorSymbol DomainSymbol RandomNumberGenerator '
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksUniqueID methodsFor: 'accessing' stamp: 'pdf 3/10/2000 07:35'!
timestamp
  timestamp isNil ifTrue: [timestamp := (creationDateAndTime at: 1) asSeconds + (creationDateAndTime at: 2) asSeconds].
  ^timestamp! !


!LinksUniqueID methodsFor: 'printing' stamp: 'pdf 3/10/2000 07:47'!
printOn: aStream 
	"Append to the argument, aStream, a sequence of characters that 
	identifies the receiver."

    self storeOn: aStream
! !

!LinksUniqueID methodsFor: 'printing' stamp: 'pdf 3/10/2000 07:45'!
storeOn: aStream 
	"Append to the argument, aStream, a sequence of characters that 
	identifies the receiver."

    aStream nextPutAll: '(', self class name.
    aStream nextPutAll: ' d: '.
    aStream nextPutAll: domainSymbol storeString.
    aStream nextPutAll: ' c: '.
    aStream nextPutAll: creatorSymbol storeString.
    aStream nextPutAll: ' t: '.
    aStream nextPutAll: self timestamp storeString.
    aStream nextPutAll: ' r: '.
    aStream nextPutAll: randomNumber storeString.
    aStream nextPutAll: ' v: '.
    aStream nextPutAll: version storeString.
    aStream nextPutAll: ')'.
! !


!LinksUniqueID methodsFor: 'initializing-releasing' stamp: 'pdf 3/10/2000 07:33'!
domainSymbol: aDomain creatorSymbol: aCreator timestamp: aTimestamp randomNumber: aRandomNumber version: aVersion
  domainSymbol := aDomain.
  creatorSymbol := aCreator.
  timestamp := aTimestamp.
  randomNumber := aRandomNumber.
  version := aVersion! !

!LinksUniqueID methodsFor: 'initializing-releasing' stamp: 'pdf 3/10/2000 07:16'!
generateCreationDateAndTime

  "^Time dateAndTimeNow."
  ^Time totalSeconds
! !

!LinksUniqueID methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:02'!
generateCreatorSymbol

  ^self class generateCreatorSymbol
! !

!LinksUniqueID methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:02'!
generateDomainSymbol

  ^self class generateDomainSymbol
! !

!LinksUniqueID methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:02'!
generateRandomNumber

   ^self class generateRandomNumber
! !

!LinksUniqueID methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 08:02'!
initialize
  domainSymbol := self generateDomainSymbol.
  creatorSymbol := self generateCreatorSymbol.
  creationDateAndTime := self generateCreationDateAndTime.
  randomNumber := self generateRandomNumber.
  version := 0. "for handling import collisions".
  
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksUniqueID class
	instanceVariableNames: ''!

!LinksUniqueID class methodsFor: 'instance creating' stamp: 'pdf 3/10/2000 07:37'!
d: aDomain c: aCreator t: aTimestamp r: aRandomNumber v: aVersion
  ^super new domainSymbol: aDomain creatorSymbol: aCreator timestamp: aTimestamp randomNumber: aRandomNumber version: aVersion; yourself.! !

!LinksUniqueID class methodsFor: 'instance creating' stamp: 'pdf 3/10/2000 07:37'!
domainSymbol: aDomain creatorSymbol: aCreator timestamp: aTimestamp randomNumber: aRandomNumber version: aVersion
  ^super new domainSymbol: aDomain creatorSymbol: aCreator timestamp: aTimestamp randomNumber: aRandomNumber version: aVersion; yourself! !

!LinksUniqueID class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:02'!
generateCreatorSymbol

   ^CreatorSymbol
! !

!LinksUniqueID class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:02'!
generateDomainSymbol

   ^DomainSymbol
! !

!LinksUniqueID class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:02'!
generateRandomNumber

   ^RandomNumberGenerator next.
! !

!LinksUniqueID class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:02'!
new

^super new initialize! !


!LinksUniqueID class methodsFor: 'class initalizing' stamp: 'pdf 3/9/2000 08:02'!
initialize

  "You should update these constants for your installation and re-initialize this class to avoid collisions"
  "LinksUniqueID initialize"

  RandomNumberGenerator := Random new.
  DomainSymbol := 'www.yourdomain.com' asSymbol.  
  CreatorSymbol := 'YourName' asSymbol.

  "For testing: "   
     "ModUniqueID new."

  "For example: I would use:
    RandomNumberGenerator := Random new.
    DomainSymbol := 'www.kurtz-fernhout.com' asSymbol.  
    CreatorSymbol := 'PaulD.Fernhout' asSymbol."
! !


LinksModelPrototype subclass: #LinksWidgetPrototype
	instanceVariableNames: 'world position extent links channels backgroundColor foregroundColor borderColor '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksWidgetPrototype methodsFor: 'connecting' stamp: 'pdf 3/9/2000 16:21'!
addLink: aLink

 self invalidateBounds.
 links add: aLink.
 self invalidateBounds.
! !

!LinksWidgetPrototype methodsFor: 'connecting' stamp: 'pdf 3/9/2000 18:35'!
positionForOffset: offset
  "deafult is to assume it is a position"
  "^self bounds center + offset "
  ^self position + offset ! !

!LinksWidgetPrototype methodsFor: 'connecting' stamp: 'pdf 3/10/2000 08:30'!
recurseLinks: aSet from: aWidget type: aSymbol
	(aSet includes: self)
		ifFalse: 
			[aSet add: self.
			self links do: [:each | each recurseLinks: aSet from: self type: aSymbol]]! !


!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:31'!
backgroundColor

^backgroundColor! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:31'!
backgroundColor: aColor
  self invalidateBounds.
  backgroundColor := aColor.
  self invalidateBounds.
  self changed: #backgroundColor.
! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 17:54'!
bounds

  ^position extent: self extent! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 18:13'!
domain
  ^world! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 17:52'!
extent
  extent isNil ifTrue: [extent := 50 @ 50].
  ^extent! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 17:51'!
extent: aPoint
  self invalidateBounds.
  extent := aPoint.
  self invalidateBounds.
  self changed: #extent.
! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/10/2000 08:14'!
links
  links isNil ifTrue: [links := OrderedCollection new].
  ^links! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/10/2000 08:31'!
lockedProperties
  "kludge for now"
  lockedProperties isNil ifTrue: [lockedProperties := IdentityDictionary new].
  ^lockedProperties.! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:31'!
position
  ^position! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:31'!
position: aPosition
  self invalidateBounds.
  position := aPosition.
  self invalidateBounds.
  self changed: #position.
! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 14:38'!
world

^world! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/10/2000 07:57'!
world: aWorld

  world := aWorld! !

!LinksWidgetPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 15:07'!
world: aWorld position: aPosition

 world := aWorld.
 position := aPosition.
 world addLink: self.! !


!LinksWidgetPrototype methodsFor: 'finding' stamp: 'pdf 3/9/2000 14:31'!
containsPoint:  aPoint

  ^self bounds containsPoint: aPoint.! !

!LinksWidgetPrototype methodsFor: 'finding' stamp: 'pdf 3/9/2000 15:12'!
widgetUnderPoint: aPoint 
	^ (self containsPoint: aPoint)
		ifTrue: [self]! !


!LinksWidgetPrototype methodsFor: 'moving' stamp: 'pdf 3/9/2000 14:55'!
moveBy: delta
  self position: position + delta.

! !


!LinksWidgetPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 14:35'!
invalidate
  world invalidate! !

!LinksWidgetPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 14:37'!
invalidate: aRect
  world isNil ifTrue: [^self].
  world invalidate: aRect! !

!LinksWidgetPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 15:57'!
invalidateBounds
  self isBoundsValid ifTrue: 
      [self invalidate: self bounds].
  links isNil ifTrue: [^self].
  links do: [:each | each invalidateBounds].! !

!LinksWidgetPrototype methodsFor: 'invalidating' stamp: 'pdf 3/9/2000 18:23'!
isBoundsValid
  ^self position notNil and: [self extent notNil].! !


!LinksWidgetPrototype methodsFor: 'displaying' stamp: 'pdf 3/9/2000 15:31'!
displayBodyOn: aCanvas invalidRect: invalidRect displayLevel: displayLevel
  
	aCanvas fillRectangle: (position extent: self bounds extent) color: backgroundColor.! !

!LinksWidgetPrototype methodsFor: 'displaying' stamp: 'pdf 3/9/2000 15:31'!
displayOn: aCanvas invalidRect: invalidRect displayLevel: displayLevel
	(self isInInvalidRect: invalidRect)
		ifFalse: [^ self].
	"Transcript show: 'displaying widget'; cr."
	self displayBodyOn: aCanvas invalidRect: invalidRect displayLevel: displayLevel.! !

!LinksWidgetPrototype methodsFor: 'displaying' stamp: 'pdf 3/9/2000 14:31'!
isInInvalidRect: aRect

  ^aRect intersects: self bounds! !

!LinksWidgetPrototype methodsFor: 'displaying' stamp: 'pdf 3/9/2000 14:31'!
isVisible
  ^true! !


!LinksWidgetPrototype methodsFor: 'streaming' stamp: 'pdf 3/10/2000 00:31'!
getID
  ^(world links indexOf: self)! !


!LinksWidgetPrototype methodsFor: 'initalizing-releasing' stamp: 'pdf 3/9/2000 15:57'!
initialize
  super initialize.
  backgroundColor := Color green.
  links := OrderedCollection new.! !


!LinksWidgetPrototype methodsFor: 'event handling' stamp: 'pdf 3/10/2000 11:19'!
handleEvent: event phase: phaseSymbol
  "(phaseSymbol ~= #processEvent or: [event processed]) ifTrue: [^self]."
  (phaseSymbol ~= #processEvent) ifTrue: [^self].
  (self containsPoint: event cursorPoint) ifTrue: ["Transcript show: 'point in bounds ', self getID printString; cr.". self invalidateBounds. event processed: true.].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksWidgetPrototype class
	instanceVariableNames: ''!

!LinksWidgetPrototype class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 14:39'!
world: world position: position

  ^self new world: world position: position; yourself.! !


!LinksWidgetPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:25'!
backgroundColor
	^ LinksProperty
		name: 'backgroundColor'
		symbol: #backgroundColor
		loadSelector: #backgroundColor
		storeSelector: #backgroundColor:
		hint: 'This is the background color for this widget.'
		type: #Color! !

!LinksWidgetPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:26'!
extent
	^ LinksProperty
		name: 'extent'
		symbol: #extent
		loadSelector: #extent
		storeSelector: #extent:
		hint: 'This is the extent for this widget.'
		type: #Point! !

!LinksWidgetPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:26'!
links
	^ LinksProperty
		name: 'links'
		symbol: #links
		loadSelector: #links
		storeSelector: nil
		hint: 'This is the list of objects (links) which hold this widget somehow.'
		type: #OrderedCollection! !

!LinksWidgetPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:26'!
position
	^ LinksProperty
		name: 'position'
		symbol: #position
		loadSelector: #position
		storeSelector: #position:
		hint: 'This is the position for this widget.'
		type: #Point! !

!LinksWidgetPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 17:51'!
propertyList
  "LinksWidgetPrototype initialize"
  ^super propertyList , #(position links backgroundColor extent)
! !


LinksWidgetPrototype subclass: #LinksWidgetLine
	instanceVariableNames: 'width '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksWidgetLine methodsFor: 'initalizing-releasing' stamp: 'pdf 3/9/2000 19:58'!
initialize
  super initialize.
  width := 3.
  backgroundColor := Color blue.! !


!LinksWidgetLine methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:57'!
bounds
  
  ^(Rectangle encompassing: (OrderedCollection with: position with: position + self extent) ) expandBy: width! !

!LinksWidgetLine methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:55'!
displayBodyOn: aCanvas invalidRect: invalidRect displayLevel: displayLevel
     
	aCanvas line: position to: position + extent width: width color: backgroundColor.! !

!LinksWidgetLine methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:54'!
width
  ^width! !

!LinksWidgetLine methodsFor: 'accessing' stamp: 'pdf 3/9/2000 20:25'!
width: aWidth
  self invalidateBounds.
  width := aWidth.
  self invalidateBounds.
  self changed: #width.! !


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksWidgetLine class
	instanceVariableNames: ''!

!LinksWidgetLine class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 19:53'!
propertyList
  "LinksWidgetLine initialize"
  ^super propertyList , #(width)
! !

!LinksWidgetLine class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:25'!
width
	^ LinksProperty
		name: 'width'
		symbol: #width
		loadSelector: #width
		storeSelector: #width:
		hint: 'This is the width for this line widget.'
		type: #Integer! !


LinksWidgetPrototype subclass: #LinksWidgetLinkPrototype
	instanceVariableNames: 'holds type '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksWidgetLinkPrototype methodsFor: 'displaying' stamp: 'pdf 3/9/2000 20:51'!
displayBodyOn: aCanvas invalidRect: invalidRect displayLevel: displayLevel 
	| previousPoint nextPoint aColor |
	displayLevel ~= #all ifTrue: [^ self].
	aColor := Color black.
	holds isEmpty ifTrue: [^ self].
	previousPoint := (holds at: 1) position.
	type == #oneWay ifTrue: [aColor := Color red].
	type == #elastic ifTrue: [aColor := Color green].
	self holds do: 
		[:each | 
		nextPoint := each position.
		aCanvas
			line: previousPoint
			to: nextPoint
			width: 1
			color: aColor.
		previousPoint := nextPoint]! !


!LinksWidgetLinkPrototype methodsFor: 'accessing' stamp: 'pdf 3/10/2000 08:16'!
addHold: widget offset: offset
  |newHold|
  newHold := LinksHold new.
  newHold widget: widget.
  newHold offset: offset.
  self holds add: newHold.
  widget addLink: self.
  self position: (holds at: 1) position.! !

!LinksWidgetLinkPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 17:00'!
bounds
  (holds isNil or: [holds size < 1]) ifTrue: [^super bounds].
  "should be cached somehow"
  ^Rectangle encompassing: (holds collect: [:each | each position]).! !

!LinksWidgetLinkPrototype methodsFor: 'accessing' stamp: 'pdf 3/10/2000 08:14'!
holds
  holds isNil ifTrue: [holds := OrderedCollection new].
  ^holds! !

!LinksWidgetLinkPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 17:13'!
type
  ^type! !

!LinksWidgetLinkPrototype methodsFor: 'accessing' stamp: 'pdf 3/9/2000 17:30'!
type: aType
  type := aType! !


!LinksWidgetLinkPrototype methodsFor: 'initalizing-releasing' stamp: 'pdf 3/9/2000 18:22'!
initialize
  super initialize.
  holds := OrderedCollection new.
  type := #solid.
  extent := 50 @ 50.! !


!LinksWidgetLinkPrototype methodsFor: 'finding' stamp: 'pdf 3/9/2000 16:27'!
containsPoint:  aPoint

  "need better test if want to interact with link"
  ^false.! !


!LinksWidgetLinkPrototype methodsFor: 'connecting' stamp: 'pdf 3/9/2000 17:27'!
fromWidget
  holds size < 1 ifTrue: [^nil].
  ^holds first widget.! !

!LinksWidgetLinkPrototype methodsFor: 'connecting' stamp: 'pdf 3/10/2000 08:29'!
recurseLinks: aSet from: aWidget type: aSymbol
     (aSymbol == #solid and: [type == #elastic]) ifTrue: [^self].
     (type == #oneWay and: [aWidget notNil and: [aWidget == self toWidget]]) ifTrue: [^self].
	(aSet includes: self)
		ifFalse: 
			[aSet add: self.
			self links do: [:each | each recurseLinks: aSet from: self type: aSymbol].
			self holds do: [:each | each widget recurseLinks: aSet from: self type: aSymbol]]! !

!LinksWidgetLinkPrototype methodsFor: 'connecting' stamp: 'pdf 3/9/2000 17:27'!
toWidget
  holds size < 2 ifTrue: [^nil].
  ^holds last widget.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksWidgetLinkPrototype class
	instanceVariableNames: ''!

!LinksWidgetLinkPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:25'!
holds
	^ LinksProperty
		name: 'holds'
		symbol: #holds
		loadSelector: #holds
		storeSelector: nil
		hint: 'This is the objects that are linked (or held on to)'
		type: #OrderedCollection! !

!LinksWidgetLinkPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 17:39'!
propertyList
  "LinksWidgetLinkPrototype initialize"
  ^super propertyList , #(type holds)
! !

!LinksWidgetLinkPrototype class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:25'!
type
	^ LinksProperty
		name: 'type'
		symbol: #type
		loadSelector: #type
		storeSelector: #type:
		hint: 'This is the type of link -- #solid, #elastic, #oneWay, etc..'
		type: #Symbol! !


LinksWidgetPrototype subclass: #LinksWidgetText
	instanceVariableNames: 'text '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksWidgetText methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:30'!
bounds

  position isNil ifTrue: [^0 @ 0 extent: 0 @ 0].
  text isNil ifTrue: [^position extent: 0 @ 0].
  ^ position extent: text asDisplayText boundingBox extent + (1 @ 1)! !

!LinksWidgetText methodsFor: 'accessing' stamp: 'pdf 3/9/2000 20:31'!
displayBodyOn: aCanvas invalidRect: invalidRect displayLevel: displayLevel

 " text displayOn: aCanvas at: position."
	"super displayBodyOn: aCanvas invalidRect: invalidRect."
" text asDisplayText displayOnPort: aCanvas at: position."
     text isNil ifTrue: [^self].
	aCanvas text: text bounds: self bounds font: nil color: Color black.
 
! !

!LinksWidgetText methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:30'!
text

^text! !

!LinksWidgetText methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:30'!
text: aText

  self invalidate: self bounds.
  text := aText.
  self invalidate: self bounds.
  self changed: #text.! !

!LinksWidgetText methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:30'!
xmlText

"need to handle xml character translation"
^text! !

!LinksWidgetText methodsFor: 'accessing' stamp: 'pdf 3/9/2000 19:30'!
xmlText: xmlText

"need to handle xml character translation"
text := xmlText! !


!LinksWidgetText methodsFor: 'streaming' stamp: 'pdf 3/9/2000 19:30'!
writeXMLForContents: aStream

super writeXMLForContents: aStream.
aStream nextPutAll: '      <text>', self xmlText,'</text>'; cr.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksWidgetText class
	instanceVariableNames: ''!

!LinksWidgetText class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 19:33'!
propertyList
  "LinksWidgetText initialize"
  ^super propertyList , #(text)
! !

!LinksWidgetText class methodsFor: 'property defining' stamp: 'pdf 3/9/2000 22:26'!
text
	^ LinksProperty
		name: 'text'
		symbol: #text
		loadSelector: #text
		storeSelector: #text:
		hint: 'This is a string of text this widget displays.'
		type: #String! !


LinksModelPrototype subclass: #LinksWorld
	instanceVariableNames: 'transactionManager links lastEvent view displayLevel '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksWorld methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:29'!
displayLevel

^displayLevel! !

!LinksWorld methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:28'!
links

^links! !


!LinksWorld methodsFor: 'displaying' stamp: 'pdf 3/9/2000 19:42'!
displayOn: aCanvas invalidRect: invalidRect
     lastEvent isNil ifTrue: [^self].
	"aCanvas fillRectangle: (lastEvent cursorPoint extent: 100 @ 100) color: Color red."
     links isNil ifTrue: [links := OrderedCollection new].
	links do: [:each | each displayOn: aCanvas invalidRect: invalidRect displayLevel: self displayLevel.].! !


!LinksWorld methodsFor: 'event handling' stamp: 'pdf 3/10/2000 11:07'!
handleEvent: event 
     " a temporary kludge to provide feedback on position"
	"Transcript show: event printString; cr."

     "invalidate old position and then new position for temp tracker object"
	"view notNil & lastEvent notNil 
          ifTrue: [view invalidate: (lastEvent cursorPoint extent: 100 @ 100)]."
	lastEvent := event.
	"view notNil ifTrue: [view invalidate: (lastEvent cursorPoint extent: 100 @ 100)]."

     "let the transaction mangager's transactions in progress do something with the event if desired"
	(transactionManager notNil and: [transactionManager handleEvent: event]) ifTrue: [^self].

     "none of the existing transactions wanted the event, so maybe make a new transaction."
	self makeTransactionForEvent: event.! !


!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 15:07'!
addLink: aLink

  links add: aLink.! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 14:55'!
addWidget: position

  |widget |
  widget := LinksWidgetPrototype world: self position: position.
  Transcript show: 'widget added'; cr.
  ^widget.! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/10/2000 07:51'!
addWidget: position special: keyValue

  |widget reply stream aXMLWriter |
  keyValue == $l asInteger 
	ifTrue: [widget := LinksWidgetLine world: self position: position].
  keyValue == $t asInteger 
	ifTrue: [widget := LinksWidgetPrototype world: self position: position].
  keyValue == $  asInteger 
	ifTrue: [widget := LinksWidgetPrototype world: self position: position].
  keyValue == $s asInteger 
	ifTrue: [
		reply := FillInTheBlank request: 'Create what string? ' initialAnswer: ''.
		reply size == 0 ifTrue: [^ self].
          widget := LinksWidgetText world: self position: position. 
	     widget text: reply.].
 keyValue == $w asInteger 
	ifTrue: [
		stream := FileStream newFileNamed: 'output.xml'.
          aXMLWriter := LinksXMLWriter on: stream.
          self writeXML: aXMLWriter.
		stream close.
		^nil.].
  keyValue == $r asInteger 
	ifTrue: [
		stream := FileStream oldFileNamed: 'output.xml'.
          self readXML: stream.
		stream close.
		^nil.].
  "Transcript show: 'widget added'; cr."
  ^widget.! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 08:24'!
displayContentsIfInvalid
  view displayContentsIfInvalid! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 08:24'!
invalidate
  "kludge for now -- should use dependencies..."
  view invalidate! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 08:24'!
invalidate: aRect
  "kludge for now -- should use dependencies..."
  view invalidate: aRect! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 08:24'!
makeTransactionForEvent: event 
     "maybe make a new transaction for the event."
	event isKeystroke 
          ifTrue: [^self makeTransactionForKeystrokeEvent: event].
	event isMouseDown 
		ifTrue: [^self makeTransactionForMouseDownEvent: event].
     "Transcript show: 'ignored event'; cr."
! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 18:18'!
makeTransactionForKeystrokeEvent: event 
	"Transcript show: 'Keystroke Event ', event keyValue printString; cr."
	"something for  debugging purposes"
	| otherWidget |
     "Put the widget at the back of all others"
	lastEvent keyValue == $- asInteger
		ifTrue: 
			[otherWidget := self widgetUnderPoint: event cursorPoint.
			otherWidget notNil
				ifTrue: 
					[self links remove: otherWidget.
					self links addFirst: otherWidget.
					self invalidate.
					^ true].
			^ true].
     "put the widget in front of all others"
	lastEvent keyValue == $+ asInteger
		ifTrue: 
			[otherWidget := self widgetUnderPoint: event cursorPoint.
			otherWidget notNil
				ifTrue: 
					[self links remove: otherWidget.
					self links add: otherWidget.
					self invalidate.
					^ true].
			^ true].
	lastEvent keyValue == $i asInteger
		ifTrue: 
			[otherWidget := self widgetUnderPoint: event cursorPoint.
			otherWidget notNil
				ifTrue: 
					[otherWidget inspect.
					self halt.
					^ true].
			self inspect.
			self halt.
			^ true].
	"this maybe should be an event -- right now, GUI state"
	lastEvent keyValue == $q asInteger
		ifTrue: 
			[displayLevel == #all
				ifTrue: [displayLevel := #some]
				ifFalse: [displayLevel := #all].
			self invalidate.
			^ true].
	"just for fun, testing with old way...."
	otherWidget := self addWidget: lastEvent cursorPoint special: lastEvent keyValue.
	otherWidget notNil ifTrue: [self invalidate: otherWidget bounds].
	^ true! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 16:45'!
makeTransactionForMouseDownEvent: event 
	"Transcript show: 'Mouse Event ', event cursorPoint printString; cr."
	"handle control key for connections"
	| newTransaction |
	event controlKeyPressed
		ifTrue: [Transcript show: 'control'; cr. newTransaction := LinksTransactionWidgetConnect manager: transactionManager]
		ifFalse: ["handle regular drag -- of either widget or entire network"
			"check if want to move the entire network or just a widget."
			newTransaction := event shiftPressed
						ifTrue: [LinksTransactionWidgetMove manager: transactionManager]
						ifFalse: [LinksTransactionNetworkMove manager: transactionManager]].
	"Transcript show: 'making transaction ', event shiftPressed  printString, ' ', newTransaction class printString; cr. "
	^ transactionManager newTransactionInProgress: newTransaction event: event! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 08:24'!
newNetwork

  |network|
  network := ModNetwork domain: self.
  networks add: network.
  ^network.! !

!LinksWorld methodsFor: 'old event handling' stamp: 'pdf 3/9/2000 08:24'!
oldHandleEvent: event 
	| otherWidget |
	"Transcript show: event printString; cr."
	view notNil & lastEvent notNil 
          ifTrue: [view invalidate: (lastEvent cursorPoint extent: 100 @ 100)].
	lastEvent := event.
	view notNil ifTrue: [view invalidate: (lastEvent cursorPoint extent: 100 @ 100)].
	lastEvent isKeystroke ifTrue: 
		["Transcript show: lastEvent keyValue printString; cr."
          lastEvent keyValue == $i asInteger ifTrue: [self inspect. self halt.].
		lastEvent keyValue == $q asInteger
			ifTrue: [showConnections := (showConnections == true) not. view invalidate.]
			ifFalse: [otherWidget := self addWidget: lastEvent cursorPoint special: lastEvent keyValue. 
					otherWidget notNil ifTrue: [view invalidate: otherWidget bounds.]].
		].
	lastEvent isMouseDown & lastEvent shiftPressed not
		ifTrue: 
			[carriedWidget := self widgetUnderPoint: lastEvent cursorPoint.
			carriedWidget notNil 
                    ifTrue: [carriedWidgetOffset := carriedWidget position - lastEvent cursorPoint].
			carriedWidgetSolo := lastEvent controlKeyPressed].
	lastEvent isMouseDown & lastEvent shiftPressed
		ifTrue: [connectingWidget := self widgetUnderPoint: lastEvent cursorPoint.].
	lastEvent isMouseMove & carriedWidget notNil
		ifTrue: 
			[carriedWidget network widgets do: [:each | view invalidate: each bounds.].
			carriedWidgetSolo 
				ifTrue: [carriedWidget position: lastEvent cursorPoint + carriedWidgetOffset.]
				ifFalse: [carriedWidget moveTo: lastEvent cursorPoint + carriedWidgetOffset.].
			carriedWidget network widgets do: [:each | view invalidate: each bounds.]].
	lastEvent isMouseUp ifTrue: 
			[carriedWidget := nil.
              connectingWidget notNil ifTrue: 
				[otherWidget := self widgetUnderPoint: lastEvent cursorPoint.
				otherWidget notNil & (otherWidget == connectingWidget) not
                       ifTrue: 
                          [connectingWidget connectTo: otherWidget.
                           view invalidate: connectingWidget bounds.
                           view invalidate: otherWidget bounds.
				        connectingWidget := nil.]
                    ]
              ].! !


!LinksWorld methodsFor: 'finding' stamp: 'pdf 3/9/2000 14:27'!
widgetUnderPoint: aPoint 
	| widget |
	links reverseDo: 
		[:each | 
		widget := each widgetUnderPoint: aPoint.
		widget notNil ifTrue: [^ widget]].
	^ nil! !


!LinksWorld methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 14:23'!
initialize
     super initialize. 
	transactionManager := LinksManagerForTransactions domain: self.
	links := OrderedCollection new.
     lastEvent := LinksEvent new.
	displayLevel := #all.! !

!LinksWorld methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 14:27'!
view: aView 
	view := aView! !


!LinksWorld methodsFor: 'streaming' stamp: 'pdf 3/9/2000 08:26'!
readXML: aStream

| reader root version |
"asumed just adding to exisitng world"
reader := LinksXMLReader on: aStream.
root := reader readAllIntoElementTree.
version := root attributes at: 'version'.
version ~= '2.0' ifTrue: [self halt: 'Can only read version 2.0, not version ', version].
self readXMLElement: root.
view invalidate.
! !

!LinksWorld methodsFor: 'streaming' stamp: 'pdf 3/10/2000 11:51'!
readXMLElement: element 
	| newWidget index linksElement linkWidget holdsElement holds heldWidget localLinks |
	element elements do: 
		[:each | 
		newWidget := (Smalltalk at: (each attributes at: 'class') asSymbol) basicNew.
		newWidget world: self.
		newWidget readXMLElement: each.
		links add: newWidget].
	"now fixup links and holds"
	index := 0.
	element elements do: 
		[:each | 
		index := index + 1.
		newWidget := links at: index.
		linksElement := each findSubElement: 'links'.
		localLinks := LinksXMLReader collectionForNumberList: linksElement content.
		localLinks do: 
			[:linkIndex | 
			linkWidget := links at: linkIndex.
			newWidget links add: linkWidget].
		holdsElement := each findSubElement: 'holds'.
		holdsElement notNil
			ifTrue: 
				[holds := LinksXMLReader collectionForNumberList: holdsElement content.
				holds do: 
					[:holdsIndex | 
					heldWidget := links at: holdsIndex.
					newWidget addHold: heldWidget offset: 0 @ 0]]]! !

!LinksWorld methodsFor: 'streaming' stamp: 'pdf 3/10/2000 00:13'!
writeXML: aXMLWriter

aXMLWriter tagStartOpen: 'world'.
aXMLWriter tagStartParameter: 'version' value: '2.0'.
aXMLWriter tagStartClose.
links do: [:each | each writeXML: aXMLWriter].
aXMLWriter tagEnd: 'world'.! !


!LinksWorld methodsFor: 'inspecting' stamp: 'pdf 3/9/2000 17:35'!
inspect
  self basicInspect! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksWorld class
	instanceVariableNames: ''!

!LinksWorld class methodsFor: 'instance creation' stamp: 'pdf 3/9/2000 08:26'!
open
	| topView newWorld|
	"Open a view of an instance of me."
	"ModWorld open."

     newWorld := self new.

	topView _ LinksSystemView new.
	topView
		model: newWorld;
		label: 'Links World';
		minimumSize: 200@200.
	topView borderWidth: 1.

      "pdf - temp kludge of embedding view in model"
     topView model view: topView.

	topView controller open.

     "Squeak actually never seems to return from this..."
     ^newWorld.
! !


Model subclass: #LinksXMLElement
	instanceVariableNames: 'tag attributes content elements '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksXMLElement methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:17'!
attributes 

^attributes
! !

!LinksXMLElement methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:17'!
content 

content isNil ifTrue: [^''].
^content
! !

!LinksXMLElement methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:17'!
elements

elements isNil ifTrue: [^#()].
^elements
! !

!LinksXMLElement methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:17'!
tag 

^tag
! !


!LinksXMLElement methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:17'!
confirmOrFail: aBoolean

"should really throw exception"
aBoolean ifFalse: [self halt.].
! !

!LinksXMLElement methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:17'!
readXML: aReader

"assumes tag is in play"
self confirmOrFail: aReader isStartTag.
tag := aReader tagName.
attributes := aReader tagAttributes.
aReader readNextThingSkipComments.
aReader isStartTag
	ifTrue: [self readXMLSubElements: aReader].
aReader isContent
	ifTrue: [content := aReader content. aReader readNextThingSkipComments.].
"needs to be at end"
self confirmOrFail: aReader isEndTag & (aReader tagName = tag)

! !

!LinksXMLElement methodsFor: 'reading' stamp: 'pdf 3/10/2000 11:51'!
readXMLSubElements: aReader
  "read sub elements -- first has been already read - recurses"
  | newElement |
  elements := OrderedCollection new.
  [aReader isAtEnd | aReader isEndTag] whileFalse:
    [
    newElement := LinksXMLElement new.
    newElement readXML: aReader.
    elements add: newElement.
    aReader readNextThingSkipComments.
    ].
  
! !


!LinksXMLElement methodsFor: 'finding' stamp: 'pdf 3/9/2000 08:17'!
findSubElement: aString

  elements do: [:each | each tag = aString ifTrue: [^each]].
  ^nil.! !


Model subclass: #LinksXMLReader
	instanceVariableNames: 'stream tagName tagAttributes negated content lastType '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
content

^content! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
isAtEnd

  ^lastType == #atEnd! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
isComment

  ^lastType == #comment! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
isContent

  ^lastType == #content! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
isEndTag

  ^lastType == #endTag! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
isStartTag

  ^lastType == #startTag! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
lastType

^lastType! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
stream

^stream! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
stream: aStream

stream := aStream! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
tagAttributes

^tagAttributes! !

!LinksXMLReader methodsFor: 'accessing' stamp: 'pdf 3/9/2000 08:18'!
tagName

^tagName! !


!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
printState

Transcript show: lastType printString, ' ', tagName printString, ' ', content printString, ' ', tagAttributes printString; cr.

! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
putBack: count
  "move stream backwards by count".
  stream atEnd | (stream position == 0) 
     ifFalse: [stream position: stream position - 1].


! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readAll

[stream atEnd] whileFalse: 
	[self readNextThing.
	self printState.].
! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/10/2000 11:51'!
readAllIntoElementTree

|root|
root := LinksXMLElement new.
self readNextThingSkipComments.
root readXML: self.
^root.
! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readAttributeValue

  | result |
  stream peek = $" ifFalse: [^''].
  stream next.
  result := stream upTo: $".
  ^result.
  
 ! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readComment

  stream upToAll: '-->'.
  "not sure if this is needed -- bugs? stream atEnd ifFalse: [stream next: 3]."
  lastType := #comment.
! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readContent

  content := stream upTo: $<.
  self putBack: 1.
  lastType := #content.
  


! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readName

 | nameStream |
 nameStream := ReadWriteStream on: String new.
 [stream peek isAlphaNumeric] 
	whileTrue: [nameStream nextPut: stream next].
  ^nameStream contents.! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readNextThing
	| c |
     self resetTagInfo.
     self skipSpaces.
	c := stream peek.
     "Transcript show: c printString; cr."
	c isNil ifTrue: [lastType := #atEnd. ^ nil].
	c = $<
		ifTrue: [self readTagOrComment]
		ifFalse: [self readContent]! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readNextThingSkipComments
	self readNextThing.
	"self printState."
     [self isComment] whileTrue: [self readNextThing. "self printState."].! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readTag
	"we've seen a < and peek-ed something other than a !!.  Parse and return a tag"
	| attributeName attributeValue c |
	
	tagAttributes _ Dictionary new.

	"determine if it's negated"
	stream peek = $/ 
		ifTrue: [ negated := true.  stream next. ]
		ifFalse: [ negated := false ].

	"read in the name"
	self skipSpaces.
	tagName := self readName.
	tagName := tagName asLowercase.

	"read in any attributes"
	[ 	self skipSpaces.
		c := stream peek.
		c = nil or: [c isLetter not ]
	] whileFalse: [
		attributeName := self readName.
		attributeName := attributeName asLowercase.
		self skipSpaces.
		stream peek = $= 
			ifTrue: [ 
				stream next.
				self skipSpaces.
				attributeValue := self readAttributeValue withoutQuoting  ]
			ifFalse: [ attributeValue := '' ].
		tagAttributes at: attributeName  put: attributeValue ].

	stream peek = $> ifTrue: [ stream next ].
     negated ifTrue: [lastType := #endTag] ifFalse: [lastType := #startTag].



! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
readTagOrComment
	"skip the $<"
  	stream next.  

	stream peek = $!! 
		ifTrue: [ self readComment ]
          ifFalse: [self readTag].

! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
resetTagInfo
     lastType := nil.
     content := nil.
     tagName := nil.
     tagAttributes := nil.
     negated := nil.
! !

!LinksXMLReader methodsFor: 'reading' stamp: 'pdf 3/9/2000 08:18'!
skipSpaces

  [Character separators includes: stream peek ] whileTrue: [stream next].

 ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksXMLReader class
	instanceVariableNames: ''!

!LinksXMLReader class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 08:18'!
on: aStream

  ^self new stream: aStream; yourself.! !

!LinksXMLReader class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 22:48'!
test
 
    "LinksXMLReader test"
    | stream reader |
     Transcript cr; show: '--------- tests ------------'; cr.
    stream := FileStream fileNamed: 'Simple.xml'.
    reader := LinksXMLReader on:  stream.
    reader readAll.
    stream close.! !

!LinksXMLReader class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 22:48'!
test2
 
    "LinksXMLReader test2"
    | stream reader result |
     Transcript cr; show: '--------- tests ------------'; cr.
    "stream := FileStream fileNamed: 'Simple.xml'."
    stream := FileStream fileNamed: 'FirstLinksWorld.xml'.
    reader := LinksXMLReader on:  stream.
    result := reader readAllIntoElementTree.
    stream close.
    ^result.! !


!LinksXMLReader class methodsFor: 'utility' stamp: 'pdf 3/9/2000 08:18'!
collectionForNumberList: aString

  |result tokens |
  aString isEmpty ifTrue: [^#()].
  tokens := aString findTokens: ' ,@'.
  result := OrderedCollection new.
  tokens do: [:each | result add: each asNumber].
  ^result.
  ! !


Model subclass: #LinksXMLWriter
	instanceVariableNames: 'stream tagStack '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Links-Core'!

!LinksXMLWriter methodsFor: 'initializing-releasing' stamp: 'pdf 3/9/2000 22:57'!
initialize
  tagStack := OrderedCollection new.
  ! !


!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:28'!
contents: aString
  stream nextPutAll: (self xmlStringFor: aString).! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/10/2000 00:16'!
indent
  1 to: tagStack size * 2 do: [:each | stream nextPut: $ ].! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:05'!
tagEnd: aTag
  self tagStackPop: aTag.
  self indent.
  stream nextPutAll: '</'.
  stream nextPutAll: aTag.
  stream nextPutAll: '>'.
  stream cr.! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/10/2000 00:26'!
tagEndSingleLine: aTag
  self tagStackPop: aTag.
  stream nextPutAll: '</'.
  stream nextPutAll: aTag.
  stream nextPutAll: '>'.
  stream cr.! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:08'!
tagStart: aTag
  self tagStart: aTag parameterString: nil.! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:08'!
tagStart: aTag parameterString: parameterString 
	self indent.
	self tagStackPush: aTag.
	stream nextPutAll: '<'.
	stream nextPutAll: aTag.
	(parameterString notNil and: [parameterString isEmpty not])
		ifTrue: 
			[stream nextPutAll: ' '.
			stream nextPutAll: parameterString].
	stream nextPutAll: '>'.
	stream cr! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:24'!
tagStartClose
  stream nextPutAll: '>'; cr.! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:25'!
tagStartOpen: tagString
   	self indent.
	self tagStackPush: tagString.
	stream nextPutAll: '<'.
	stream nextPutAll: tagString.! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:27'!
tagStartParameter: name value: value
  stream nextPutAll: ' '.
  stream nextPutAll: name.
  stream nextPutAll: '="'.
  stream nextPutAll: (self xmlStringFor: value).
  stream nextPutAll: '"'.
  ! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/10/2000 00:23'!
tagStartSingleLine: aTag
	self indent.
	self tagStackPush: aTag.
	stream nextPutAll: '<'.
	stream nextPutAll: aTag.
	stream nextPutAll: '>'.! !

!LinksXMLWriter methodsFor: 'writing' stamp: 'pdf 3/9/2000 23:28'!
xmlStringFor: aString
  "unfinished -- need to handle embedded quotes and & etc."
  ^aString! !


!LinksXMLWriter methodsFor: 'accessing' stamp: 'pdf 3/10/2000 00:15'!
stream: aStream
  stream := aStream.! !

!LinksXMLWriter methodsFor: 'accessing' stamp: 'pdf 3/9/2000 22:55'!
tagStackDepth
  ^tagStack size! !

!LinksXMLWriter methodsFor: 'accessing' stamp: 'pdf 3/9/2000 22:55'!
tagStackPop
  tagStack removeLast! !

!LinksXMLWriter methodsFor: 'accessing' stamp: 'pdf 3/9/2000 22:57'!
tagStackPop: aSymbol
  tagStack last ~= aSymbol ifTrue: [self halt: 'unexpected tag on top of stack'].
  tagStack removeLast! !

!LinksXMLWriter methodsFor: 'accessing' stamp: 'pdf 3/9/2000 22:56'!
tagStackPush: aSymbol
  tagStack add: aSymbol! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinksXMLWriter class
	instanceVariableNames: ''!

!LinksXMLWriter class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 22:58'!
new
  ^super new initialize; yourself! !

!LinksXMLWriter class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 22:53'!
on: aStream

  ^self new stream: aStream; yourself.! !

!LinksXMLWriter class methodsFor: 'instance creating' stamp: 'pdf 3/9/2000 22:59'!
test: aWorld
 
    "LinksXMLWriter test: aWorld"
    | stream writer |
     Transcript cr; show: '--------- tests ------------'; cr.
    stream := FileStream fileNamed: 'Simple.xml'.
    writer := LinksXMLWriter on:  stream.
    writer write: aWorld
    stream close.! !

LinksModelPrototype initialize!
LinksUniqueID initialize!